import pytest
from tests.mocks import MockOktaClient
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError
import okta.models as models


class TestIdentityProvidersResource:
    """
    Integration Tests for the Identity Providers Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_add_get_generic_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        ISSUER_URL = "https://idp.example.com"
        idp_model = models.IdentityProvider({
            "type": "OIDC",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} generic",
            "protocol": models.Protocol({
                "algorithms": models.ProtocolAlgorithms({
                    "request": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "REQUEST"
                        })
                    }),
                    "response": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "ANY"
                        })
                    })
                }),
                "endpoints": models.ProtocolEndpoints({
                    "acs": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "type": "INSTANCE"
                    }),
                    "authorization": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/authorize"
                    }),
                    "token": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "url": ISSUER_URL + "/token"
                    }),
                    "userInfo": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/userinfo"
                    }),
                    "jwks": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/keys"
                    })
                }),
                "scopes": ["openid", "profile", "email"],
                "type": "OIDC",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                }),
                "issuer": models.ProtocolEndpoint({
                    "url": ISSUER_URL
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.name == idp_model.name
        assert retrieved_idp.type == idp_model.type
        assert retrieved_idp.status == "ACTIVE"
        prot_endp = retrieved_idp.protocol.endpoints
        assert prot_endp.authorization.url == ISSUER_URL + "/authorize"
        assert prot_endp.authorization.binding == "HTTP-REDIRECT"
        assert prot_endp.token.url == ISSUER_URL + "/token"
        assert prot_endp.token.binding == "HTTP-POST"
        assert prot_endp.user_info.url == ISSUER_URL + "/userinfo"
        assert prot_endp.user_info.binding == "HTTP-REDIRECT"
        assert prot_endp.jwks.url == ISSUER_URL + "/keys"
        assert prot_endp.jwks.binding == "HTTP-REDIRECT"
        assert set(retrieved_idp.protocol.scopes) == set(
            idp_model.protocol.scopes)
        assert retrieved_idp.protocol.issuer.url == ISSUER_URL
        assert retrieved_idp.protocol.credentials.client.client_id ==\
            "your-client-id"
        assert retrieved_idp.protocol.credentials.client.client_secret ==\
            "your-client-secret"
        prov = retrieved_idp.policy.provisioning
        assert prov.action == "AUTO"
        assert prov.profile_master is False
        assert prov.groups.action == "NONE"
        assert prov.conditions.deprovisioned.action == "NONE"
        assert prov.conditions.suspended.action == "NONE"
        assert retrieved_idp.policy.account_link.action == "AUTO"
        subj = retrieved_idp.policy.subject
        assert subj.user_name_template.template == "idpuser.email"
        assert subj.filter is None
        assert subj.match_type == models.PolicySubjectMatchType.USERNAME
        assert subj.match_attribute is None

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_add_get_facebook_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.name == idp_model.name
        assert retrieved_idp.type == idp_model.type
        assert retrieved_idp.status == "ACTIVE"
        assert set(retrieved_idp.protocol.scopes) == set(
            idp_model.protocol.scopes)
        assert retrieved_idp.protocol.credentials.client.client_id ==\
            "your-client-id"
        assert retrieved_idp.protocol.credentials.client.client_secret ==\
            "your-client-secret"
        prov = retrieved_idp.policy.provisioning
        assert prov.action == "AUTO"
        assert prov.profile_master is True
        assert prov.groups.action == "NONE"
        assert prov.conditions.deprovisioned.action == "NONE"
        assert prov.conditions.suspended.action == "NONE"
        assert retrieved_idp.policy.account_link.action == "AUTO"
        subj = retrieved_idp.policy.subject
        assert subj.user_name_template.template == "idpuser.email"
        assert subj.filter is None
        assert subj.match_type == models.PolicySubjectMatchType.USERNAME

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_add_get_linkedin_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "LINKEDIN",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["r_basicprofile", "r_emailaddress"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.name == idp_model.name
        assert retrieved_idp.type == idp_model.type
        assert retrieved_idp.status == "ACTIVE"
        assert set(retrieved_idp.protocol.scopes) == set(
            idp_model.protocol.scopes)
        assert retrieved_idp.protocol.credentials.client.client_id ==\
            "your-client-id"
        assert retrieved_idp.protocol.credentials.client.client_secret ==\
            "your-client-secret"
        prov = retrieved_idp.policy.provisioning
        assert prov.action == "AUTO"
        assert prov.profile_master is True
        assert prov.groups.action == "NONE"
        assert prov.conditions.deprovisioned.action == "NONE"
        assert prov.conditions.suspended.action == "NONE"
        assert retrieved_idp.policy.account_link.action == "AUTO"
        subj = retrieved_idp.policy.subject
        assert subj.user_name_template.template == "idpuser.email"
        assert subj.filter is None
        assert subj.match_type == models.PolicySubjectMatchType.USERNAME

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_idps(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        ISSUER_URL = "https://idp.example.com"
        idp_model = models.IdentityProvider({
            "type": "OIDC",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} generic",
            "protocol": models.Protocol({
                "algorithms": models.ProtocolAlgorithms({
                    "request": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "REQUEST"
                        })
                    }),
                    "response": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "ANY"
                        })
                    })
                }),
                "endpoints": models.ProtocolEndpoints({
                    "acs": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "type": "INSTANCE"
                    }),
                    "authorization": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/authorize"
                    }),
                    "token": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "url": ISSUER_URL + "/token"
                    }),
                    "userInfo": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/userinfo"
                    }),
                    "jwks": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/keys"
                    })
                }),
                "scopes": ["openid", "profile", "email"],
                "type": "OIDC",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                }),
                "issuer": models.ProtocolEndpoint({
                    "url": ISSUER_URL
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # List IDPs
        idp_list, _, err = await client.list_identity_providers()
        assert err is None
        assert isinstance(idp_list, list)
        assert next((idp for idp in idp_list
                     if idp.id == created_idp.id), None) is not None

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_activate_deactivate_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        ISSUER_URL = "https://idp.example.com"
        idp_model = models.IdentityProvider({
            "type": "OIDC",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} generic",
            "protocol": models.Protocol({
                "algorithms": models.ProtocolAlgorithms({
                    "request": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "REQUEST"
                        })
                    }),
                    "response": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "ANY"
                        })
                    })
                }),
                "endpoints": models.ProtocolEndpoints({
                    "acs": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "type": "INSTANCE"
                    }),
                    "authorization": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/authorize"
                    }),
                    "token": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "url": ISSUER_URL + "/token"
                    }),
                    "userInfo": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/userinfo"
                    }),
                    "jwks": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/keys"
                    })
                }),
                "scopes": ["openid", "profile", "email"],
                "type": "OIDC",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                }),
                "issuer": models.ProtocolEndpoint({
                    "url": ISSUER_URL
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.status == "ACTIVE"

        # Deactivate
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.status == "INACTIVE"

        # Reactivate
        reactivated_idp, _, err = await \
            client.activate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(reactivated_idp, models.IdentityProvider)

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.status == "ACTIVE"

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        ISSUER_URL = "https://idp.example.com"
        idp_model = models.IdentityProvider({
            "type": "OIDC",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} generic",
            "protocol": models.Protocol({
                "algorithms": models.ProtocolAlgorithms({
                    "request": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "REQUEST"
                        })
                    }),
                    "response": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "ANY"
                        })
                    })
                }),
                "endpoints": models.ProtocolEndpoints({
                    "acs": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "type": "INSTANCE"
                    }),
                    "authorization": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/authorize"
                    }),
                    "token": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "url": ISSUER_URL + "/token"
                    }),
                    "userInfo": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/userinfo"
                    }),
                    "jwks": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/keys"
                    })
                }),
                "scopes": ["openid", "profile", "email"],
                "type": "OIDC",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                }),
                "issuer": models.ProtocolEndpoint({
                    "url": ISSUER_URL
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.status == "ACTIVE"

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

        # Retrieve for verification
        retrieved_idp, resp, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is not None
        assert isinstance(err, OktaAPIError)
        assert resp.get_status() == HTTPStatus.NOT_FOUND
        assert retrieved_idp is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_idp(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        ISSUER_URL = "https://idp.example.com"
        idp_model = models.IdentityProvider({
            "type": "OIDC",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} generic",
            "protocol": models.Protocol({
                "algorithms": models.ProtocolAlgorithms({
                    "request": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "REQUEST"
                        })
                    }),
                    "response": models.ProtocolAlgorithmType({
                        "signature": models.ProtocolAlgorithmTypeSignature({
                            "algorithm": "SHA-256",
                            "scope": "ANY"
                        })
                    })
                }),
                "endpoints": models.ProtocolEndpoints({
                    "acs": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "type": "INSTANCE"
                    }),
                    "authorization": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/authorize"
                    }),
                    "token": models.ProtocolEndpoint({
                        "binding": "HTTP-POST",
                        "url": ISSUER_URL + "/token"
                    }),
                    "userInfo": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/userinfo"
                    }),
                    "jwks": models.ProtocolEndpoint({
                        "binding": "HTTP-REDIRECT",
                        "url": ISSUER_URL + "/keys"
                    })
                }),
                "scopes": ["openid", "profile", "email"],
                "type": "OIDC",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                }),
                "issuer": models.ProtocolEndpoint({
                    "url": ISSUER_URL
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.status == "ACTIVE"

        # Update
        created_idp.name = created_idp.name + "UPDATE"
        updated_idp, _, err = await \
            client.update_identity_provider(created_idp.id, created_idp)
        assert err is None
        assert isinstance(updated_idp, models.IdentityProvider)
        assert updated_idp.id == created_idp.id
        assert updated_idp.name != idp_model.name

        # Retrieve for verification
        retrieved_idp, _, err = await client.\
            get_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(retrieved_idp, models.IdentityProvider)
        assert retrieved_idp.id == created_idp.id
        assert retrieved_idp.name == idp_model.name + "UPDATE"

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_get_key(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Key
        key = "MIIDnjCCAoagAwIBAgIGAVG3MN+PMA0GCSqGSIb3DQEBBQUAMIGPMQswCQYDV\
            QQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEWMBQGA1UEBwwNU2FuIEZyYW5ja\
            XNjbzENMAsGA1UECgwET2t0YTEUMBIGA1UECwwLU1NPUHJvdmlkZXIxEDAOBgNVB\
            AMMB2V4YW1wbGUxHDAaBgkqhkiG9w0BCQEWDWluZm9Ab2t0YS5jb20wHhcNMTUxM\
            jE4MjIyMjMyWhcNMjUxMjE4MjIyMzMyWjCBjzELMAkGA1UEBhMCVVMxEzARBgNVB\
            AgMCkNhbGlmb3JuaWExFjAUBgNVBAcMDVNhbiBGcmFuY2lzY28xDTALBgNVBAoMB\
            E9rdGExFDASBgNVBAsMC1NTT1Byb3ZpZGVyMRAwDgYDVQQDDAdleGFtcGxlMRwwG\
            gYJKoZIhvcNAQkBFg1pbmZvQG9rdGEuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCA\
            Q8AMIIBCgKCAQEAtcnyvuVCrsFEKCwHDenS3Ocjed8eWDv3zLtD2K/iZfE8BMj2w\
            pTfn6Ry8zCYey3mWlKdxIybnV9amrujGRnE0ab6Q16v9D6RlFQLOG6dwqoRKuZy3\
            3Uyg8PGdEudZjGbWuKCqqXEp+UKALJHV+k4wWeVH8g5d1n3KyR2TVajVJpCrPhLF\
            mq1Il4G/IUnPe4MvjXqB6CpKkog1+ThWsItPRJPAM+RweFHXq7KfChXsYE7Mmful\
            y8sDQlvBmQyxZnFHVuiPfCvGHJjpvHy11YlHdOjfgqHRvZbmo30+y0X/oY/yV4YE\
            J00LL6eJWU4wi7ViY3HP6/VCdRjHoRdr5L/DwIDAQABMA0GCSqGSIb3DQEBBQUAA\
            4IBAQCzzhOFkvyYLNFj2WDcq1YqD4sBy1iCia9QpRH3rjQvMKDwQDYWbi6EdOX0T\
            Q/IYR7UWGj+2pXd6v0t33lYtoKocp/4lUvT3tfBnWZ5KnObi+J2uY2teUqoYkASN\
            7F+GRPVOuMVoVgm05ss8tuMb2dLc9vsx93sDt+XlMTv/2qi5VPwaDtqduKkzwW9l\
            Ufn4xIMkTiVvCpe0X2HneD2Bpuao3/U8Rk0uiPfq6TooWaoW3kjsmErhEAs9bA7x\
            uqo1KKY9CdHcFhkSsMhoeaZylZHtzbnoipUlQKSLMdJQiiYZQ0bYL83/Ta9fulr1\
            EERICMFt3GUmtYaZZKHpWSfdJp9"

        jwk_model = models.JsonWebKey({
            "x5C": [key]
        })
        created_key, _, err = await client.\
            create_identity_provider_key(jwk_model)
        assert err is None
        assert isinstance(created_key, models.JsonWebKey)
        assert key in created_key.x_5_c

        # Retrieve
        retrieved_key, _, err = await client.\
            get_identity_provider_key(created_key.kid)
        assert err is None
        assert isinstance(retrieved_key, models.JsonWebKey)
        assert retrieved_key.kid == created_key.kid
        assert key in retrieved_key.x_5_c

        # Delete
        _, err = await client.delete_identity_provider_key(created_key.kid)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_keys(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Key
        key = "MIIDnjCCAoagAwIBAgIGAVG3MN+PMA0GCSqGSIb3DQEBBQUAMIGPMQswCQYDV\
            QQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEWMBQGA1UEBwwNU2FuIEZyYW5ja\
            XNjbzENMAsGA1UECgwET2t0YTEUMBIGA1UECwwLU1NPUHJvdmlkZXIxEDAOBgNVB\
            AMMB2V4YW1wbGUxHDAaBgkqhkiG9w0BCQEWDWluZm9Ab2t0YS5jb20wHhcNMTUxM\
            jE4MjIyMjMyWhcNMjUxMjE4MjIyMzMyWjCBjzELMAkGA1UEBhMCVVMxEzARBgNVB\
            AgMCkNhbGlmb3JuaWExFjAUBgNVBAcMDVNhbiBGcmFuY2lzY28xDTALBgNVBAoMB\
            E9rdGExFDASBgNVBAsMC1NTT1Byb3ZpZGVyMRAwDgYDVQQDDAdleGFtcGxlMRwwG\
            gYJKoZIhvcNAQkBFg1pbmZvQG9rdGEuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCA\
            Q8AMIIBCgKCAQEAtcnyvuVCrsFEKCwHDenS3Ocjed8eWDv3zLtD2K/iZfE8BMj2w\
            pTfn6Ry8zCYey3mWlKdxIybnV9amrujGRnE0ab6Q16v9D6RlFQLOG6dwqoRKuZy3\
            3Uyg8PGdEudZjGbWuKCqqXEp+UKALJHV+k4wWeVH8g5d1n3KyR2TVajVJpCrPhLF\
            mq1Il4G/IUnPe4MvjXqB6CpKkog1+ThWsItPRJPAM+RweFHXq7KfChXsYE7Mmful\
            y8sDQlvBmQyxZnFHVuiPfCvGHJjpvHy11YlHdOjfgqHRvZbmo30+y0X/oY/yV4YE\
            J00LL6eJWU4wi7ViY3HP6/VCdRjHoRdr5L/DwIDAQABMA0GCSqGSIb3DQEBBQUAA\
            4IBAQCzzhOFkvyYLNFj2WDcq1YqD4sBy1iCia9QpRH3rjQvMKDwQDYWbi6EdOX0T\
            Q/IYR7UWGj+2pXd6v0t33lYtoKocp/4lUvT3tfBnWZ5KnObi+J2uY2teUqoYkASN\
            7F+GRPVOuMVoVgm05ss8tuMb2dLc9vsx93sDt+XlMTv/2qi5VPwaDtqduKkzwW9l\
            Ufn4xIMkTiVvCpe0X2HneD2Bpuao3/U8Rk0uiPfq6TooWaoW3kjsmErhEAs9bA7x\
            uqo1KKY9CdHcFhkSsMhoeaZylZHtzbnoipUlQKSLMdJQiiYZQ0bYL83/Ta9fulr1\
            EERICMFt3GUmtYaZZKHpWSfdJp9"

        jwk_model = models.JsonWebKey({
            "x5C": [key]
        })
        created_key, _, err = await client.\
            create_identity_provider_key(jwk_model)
        assert err is None
        assert isinstance(created_key, models.JsonWebKey)
        assert key in created_key.x_5_c

        # List
        idp_keys, _, err = await client.list_identity_provider_keys()
        assert err is None
        assert isinstance(idp_keys, list)
        assert next((key for key in idp_keys if key.kid == created_key.kid),
                    None) is not None

        # Delete
        _, err = await client.delete_identity_provider_key(created_key.kid)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_key(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Key
        key = "MIIDnjCCAoagAwIBAgIGAVG3MN+PMA0GCSqGSIb3DQEBBQUAMIGPMQswCQYDV\
            QQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEWMBQGA1UEBwwNU2FuIEZyYW5ja\
            XNjbzENMAsGA1UECgwET2t0YTEUMBIGA1UECwwLU1NPUHJvdmlkZXIxEDAOBgNVB\
            AMMB2V4YW1wbGUxHDAaBgkqhkiG9w0BCQEWDWluZm9Ab2t0YS5jb20wHhcNMTUxM\
            jE4MjIyMjMyWhcNMjUxMjE4MjIyMzMyWjCBjzELMAkGA1UEBhMCVVMxEzARBgNVB\
            AgMCkNhbGlmb3JuaWExFjAUBgNVBAcMDVNhbiBGcmFuY2lzY28xDTALBgNVBAoMB\
            E9rdGExFDASBgNVBAsMC1NTT1Byb3ZpZGVyMRAwDgYDVQQDDAdleGFtcGxlMRwwG\
            gYJKoZIhvcNAQkBFg1pbmZvQG9rdGEuY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCA\
            Q8AMIIBCgKCAQEAtcnyvuVCrsFEKCwHDenS3Ocjed8eWDv3zLtD2K/iZfE8BMj2w\
            pTfn6Ry8zCYey3mWlKdxIybnV9amrujGRnE0ab6Q16v9D6RlFQLOG6dwqoRKuZy3\
            3Uyg8PGdEudZjGbWuKCqqXEp+UKALJHV+k4wWeVH8g5d1n3KyR2TVajVJpCrPhLF\
            mq1Il4G/IUnPe4MvjXqB6CpKkog1+ThWsItPRJPAM+RweFHXq7KfChXsYE7Mmful\
            y8sDQlvBmQyxZnFHVuiPfCvGHJjpvHy11YlHdOjfgqHRvZbmo30+y0X/oY/yV4YE\
            J00LL6eJWU4wi7ViY3HP6/VCdRjHoRdr5L/DwIDAQABMA0GCSqGSIb3DQEBBQUAA\
            4IBAQCzzhOFkvyYLNFj2WDcq1YqD4sBy1iCia9QpRH3rjQvMKDwQDYWbi6EdOX0T\
            Q/IYR7UWGj+2pXd6v0t33lYtoKocp/4lUvT3tfBnWZ5KnObi+J2uY2teUqoYkASN\
            7F+GRPVOuMVoVgm05ss8tuMb2dLc9vsx93sDt+XlMTv/2qi5VPwaDtqduKkzwW9l\
            Ufn4xIMkTiVvCpe0X2HneD2Bpuao3/U8Rk0uiPfq6TooWaoW3kjsmErhEAs9bA7x\
            uqo1KKY9CdHcFhkSsMhoeaZylZHtzbnoipUlQKSLMdJQiiYZQ0bYL83/Ta9fulr1\
            EERICMFt3GUmtYaZZKHpWSfdJp9"

        jwk_model = models.JsonWebKey({
            "x5C": [key]
        })
        created_key, _, err = await client.\
            create_identity_provider_key(jwk_model)
        assert err is None
        assert isinstance(created_key, models.JsonWebKey)
        assert key in created_key.x_5_c

        # Delete
        _, err = await client.delete_identity_provider_key(created_key.kid)
        assert err is None

        # Retrieve
        retrieved_key, resp, err = await client.\
            get_identity_provider_key(created_key.kid)
        assert err is not None
        assert isinstance(err, OktaAPIError)
        assert resp.get_status() == HTTPStatus.NOT_FOUND
        assert retrieved_key is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_generate_get_idp_signing_key(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Generate Key
        query_params_generate = {"validityYears": 2}
        generated_key, _, err = await client.\
            generate_identity_provider_signing_key(
                created_idp.id, query_params_generate)
        assert err is None
        assert isinstance(generated_key, models.JsonWebKey)
        assert generated_key is not None

        # Retrieve Key
        retrieved_key, _, err = await client.\
            get_identity_provider_signing_key(
                created_idp.id, generated_key.kid)
        assert err is None
        assert isinstance(retrieved_key, models.JsonWebKey)
        assert retrieved_key.kid == generated_key.kid

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_idp_signing_keys(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Generate Keys
        query_params_generate = {"validityYears": 2}
        generated_key_1, _, err = await client.\
            generate_identity_provider_signing_key(
                created_idp.id, query_params_generate)
        assert err is None
        assert isinstance(generated_key_1, models.JsonWebKey)
        assert generated_key_1 is not None

        generated_key_2, _, err = await client.\
            generate_identity_provider_signing_key(
                created_idp.id, query_params_generate)
        assert err is None
        assert isinstance(generated_key_2, models.JsonWebKey)
        assert generated_key_2 is not None

        # List Keys
        idp_signing_keys, _, err = await client.\
            list_identity_provider_signing_keys(created_idp.id)
        assert err is None
        assert isinstance(idp_signing_keys, list)
        assert next((key for key in idp_signing_keys
                     if key.kid == generated_key_1.kid), None) is not None
        assert next((key for key in idp_signing_keys
                     if key.kid == generated_key_2.kid), None) is not None

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_clone_idp_signing_key(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDPs
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        idp_model_2 = models.IdentityProvider({
            "type": "LINKEDIN",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["r_basicprofile", "r_emailaddress"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        created_idp_2, _, err = await\
            client.create_identity_provider(idp_model_2)
        assert err is None
        assert isinstance(created_idp_2, models.IdentityProvider)
        assert created_idp_2.name == idp_model.name

        # Generate Key
        query_params_generate = {"validityYears": 2}
        generated_key, _, err = await client.\
            generate_identity_provider_signing_key(
                created_idp.id, query_params_generate)
        assert err is None
        assert isinstance(generated_key, models.JsonWebKey)
        assert generated_key is not None

        # Clone Key
        query_params_clone = {"targetIdpId": created_idp_2.id}
        cloned_key, _, err = await \
            client.clone_identity_provider_key(
                created_idp.id, generated_key.kid, query_params_clone)
        assert err is None
        assert isinstance(cloned_key, models.JsonWebKey)
        assert cloned_key.kid == generated_key.kid

        # Retrieve Key
        retrieved_key, _, err = await client.\
            get_identity_provider_signing_key(
                created_idp_2.id, cloned_key.kid)
        assert err is None
        assert isinstance(retrieved_key, models.JsonWebKey)
        assert retrieved_key.kid == cloned_key.kid

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp_2.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp_2.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_generate_get_csr(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Generate CSR
        csr_metadata_model = models.CsrMetadata({
            "subject": models.CsrMetadataSubject({
                "countryName": "CAN",
                "stateOrProvinceName": "Ontario",
                "localityName": "Toronto",
                "organizationName": "Okta, Inc.",
                "organizationalUnitName": "Dev",
                "commonName": "SP Issuer"
            }),
            "subjectAltNames": models.CsrMetadataSubjectAltNames({
                "dnsNames": ["dev.okta.com"]
            })
        })
        generated_csr, _, err = await client.\
            generate_csr_for_identity_provider(
                created_idp.id, csr_metadata_model)
        assert err is None
        assert isinstance(generated_csr, models.Csr)
        assert generated_csr.kty == "RSA"

        # Retrieve
        retrieved_csr, _, err = await client.\
            get_csr_for_identity_provider(created_idp.id, generated_csr.id)
        assert err is None
        assert isinstance(retrieved_csr, models.Csr)
        assert retrieved_csr.id == generated_csr.id
        assert retrieved_csr.kty == generated_csr.kty

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_revoke_csr(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create IDP
        idp_model = models.IdentityProvider({
            "type": "FACEBOOK",
            "name": f"{TestIdentityProvidersResource.SDK_PREFIX} facebook",
            "protocol": models.Protocol({
                "scopes": ["public_profile", "email"],
                "type": "OAUTH2",
                "credentials": models.IdentityProviderCredentials({
                    "client": models.IdentityProviderCredentialsClient({
                        "clientId": "your-client-id",
                        "clientSecret": "your-client-secret"
                    })
                })
            }),
            "policy": models.IdentityProviderPolicy({
                "accountLink": models.PolicyAccountLink({
                    "action": "AUTO",
                    "filter": None
                }),
                "provisioning": models.Provisioning({
                    "action": "AUTO",
                    "profileMaster": True,
                    "conditions": models.ProvisioningConditions({
                        "deprovisioned":
                        models.ProvisioningDeprovisionedCondition({
                            "action": "NONE"
                        }),
                        "suspended": models.ProvisioningSuspendedCondition({
                            "action": "NONE"
                        })
                    }),
                    "groups": models.ProvisioningGroups({
                        "action": "NONE"
                    })
                }),
                "maxClockSkew": 120000,
                "subject": models.PolicySubject({
                    "userNameTemplate": models.PolicyUserNameTemplate({
                        "template": "idpuser.email"
                    }),
                    "matchType": models.PolicySubjectMatchType.USERNAME,
                    "filter": None
                })
            })
        })

        created_idp, _, err = await client.create_identity_provider(idp_model)
        assert err is None
        assert isinstance(created_idp, models.IdentityProvider)
        assert created_idp.name == idp_model.name

        # Generate CSR
        csr_metadata_model = models.CsrMetadata({
            "subject": models.CsrMetadataSubject({
                "countryName": "CAN",
                "stateOrProvinceName": "Ontario",
                "localityName": "Toronto",
                "organizationName": "Okta, Inc.",
                "organizationalUnitName": "Dev",
                "commonName": "SP Issuer"
            }),
            "subjectAltNames": models.CsrMetadataSubjectAltNames({
                "dnsNames": ["dev.okta.com"]
            })
        })
        generated_csr, _, err = await client.\
            generate_csr_for_identity_provider(
                created_idp.id, csr_metadata_model)
        assert err is None
        assert isinstance(generated_csr, models.Csr)
        assert generated_csr.kty == "RSA"

        # Retrieve
        retrieved_csr, _, err = await client.\
            get_csr_for_identity_provider(created_idp.id, generated_csr.id)
        assert err is None
        assert isinstance(retrieved_csr, models.Csr)
        assert retrieved_csr.id == generated_csr.id
        assert retrieved_csr.kty == generated_csr.kty

        # Revoke
        _, err = await client.\
            revoke_csr_for_identity_provider(created_idp.id, generated_csr.id)
        assert err is None

        # Retrieve for verification
        retrieved_csr, resp, err = await client.\
            get_csr_for_identity_provider(created_idp.id, generated_csr.id)
        assert err is not None
        assert isinstance(err, OktaAPIError)
        assert resp.get_status() == HTTPStatus.NOT_FOUND
        assert retrieved_csr is None

        # Deactivate and delete
        deactivated_idp, _, err = await \
            client.deactivate_identity_provider(created_idp.id)
        assert err is None
        assert isinstance(deactivated_idp, models.IdentityProvider)
        _, err = await client.delete_identity_provider(created_idp.id)
        assert err is None



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.authorization_server\
    import AuthorizationServer
from okta.models.o_auth_2_claim\
    import OAuth2Claim
from okta.models.o_auth_2_client\
    import OAuth2Client
from okta.models.o_auth_2_refresh_token\
    import OAuth2RefreshToken
from okta.models.json_web_key\
    import JsonWebKey
from okta.models.policy\
    import Policy
from okta.models.o_auth_2_scope\
    import OAuth2Scope
from okta.utils import format_url
from okta.api_client import APIClient
from okta.constants import find_policy_model


class AuthorizationServerClient(APIClient):
    """
    A Client object for the AuthorizationServer resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_authorization_servers(
            self, query_params={}
    ):
        """
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.q] {str}
            [query_params.limit] {str}
            [query_params.after] {str}
        Returns:
            list: Collection of AuthorizationServer instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, AuthorizationServer)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(AuthorizationServer(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_authorization_server(
            self, authorization_server
    ):
        """
        Args:
            {authorization_server}
        Returns:
            AuthorizationServer
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers
            """)

        if isinstance(authorization_server, dict):
            body = authorization_server
        else:
            body = authorization_server.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, AuthorizationServer)

        if error:
            return (None, response, error)

        try:
            result = AuthorizationServer(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_authorization_server(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_authorization_server(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            AuthorizationServer
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, AuthorizationServer)

        if error:
            return (None, response, error)

        try:
            result = AuthorizationServer(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_authorization_server(
            self, authServerId, authorization_server
    ):
        """
        Args:
            auth_server_id {str}
            {authorization_server}
        Returns:
            AuthorizationServer
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
            """)

        if isinstance(authorization_server, dict):
            body = authorization_server
        else:
            body = authorization_server.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, AuthorizationServer)

        if error:
            return (None, response, error)

        try:
            result = AuthorizationServer(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_o_auth_2_claims(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            list: Collection of OAuth2Claim instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/claims
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Claim)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(OAuth2Claim(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_o_auth_2_claim(
            self, authServerId, o_auth_2_claim
    ):
        """
        Args:
            auth_server_id {str}
            {o_auth_2_claim}
        Returns:
            OAuth2Claim
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/claims
            """)

        if isinstance(o_auth_2_claim, dict):
            body = o_auth_2_claim
        else:
            body = o_auth_2_claim.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Claim)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Claim(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_o_auth_2_claim(
            self, authServerId, claimId
    ):
        """
        Args:
            auth_server_id {str}
            claim_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/claims
                /{claimId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_o_auth_2_claim(
            self, authServerId, claimId
    ):
        """
        Args:
            auth_server_id {str}
            claim_id {str}
        Returns:
            OAuth2Claim
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/claims
                /{claimId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Claim)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Claim(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_o_auth_2_claim(
            self, authServerId, claimId, o_auth_2_claim
    ):
        """
        Args:
            auth_server_id {str}
            claim_id {str}
            {o_auth_2_claim}
        Returns:
            OAuth2Claim
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/claims
                /{claimId}
            """)

        if isinstance(o_auth_2_claim, dict):
            body = o_auth_2_claim
        else:
            body = o_auth_2_claim.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Claim)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Claim(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_o_auth_2_clients_for_authorization_server(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            list: Collection of OAuth2Client instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/clients
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Client)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(OAuth2Client(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def revoke_refresh_tokens_for_authorization_server_and_client(
            self, authServerId, clientId
    ):
        """
        Args:
            auth_server_id {str}
            client_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/clients
                /{clientId}/tokens
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_refresh_tokens_for_authorization_server_and_client(
            self, authServerId, clientId, query_params={}
    ):
        """
        Args:
            auth_server_id {str}
            client_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.expand] {str}
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of OAuth2RefreshToken instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/clients
                /{clientId}/tokens
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2RefreshToken)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(OAuth2RefreshToken(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def revoke_refresh_token_for_authorization_server_and_client(
            self, authServerId, clientId, tokenId
    ):
        """
        Args:
            auth_server_id {str}
            client_id {str}
            token_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/clients
                /{clientId}/tokens/{tokenId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_refresh_token_for_authorization_server_and_client(
            self, authServerId, clientId, tokenId, query_params={}
    ):
        """
        Args:
            auth_server_id {str}
            client_id {str}
            token_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.expand] {str}
        Returns:
            OAuth2RefreshToken
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/clients
                /{clientId}/tokens/{tokenId}
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2RefreshToken)

        if error:
            return (None, response, error)

        try:
            result = OAuth2RefreshToken(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_authorization_server_keys(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            list: Collection of JsonWebKey instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /credentials/keys
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, JsonWebKey)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(JsonWebKey(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def rotate_authorization_server_keys(
            self, authServerId, jwk_use
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            list: Collection of JsonWebKey instances.
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /credentials/lifecycle/keyRotate
            """)

        if isinstance(jwk_use, dict):
            body = jwk_use
        else:
            body = jwk_use.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, JsonWebKey)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(JsonWebKey(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_authorization_server(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /lifecycle/activate
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def deactivate_authorization_server(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /lifecycle/deactivate
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_authorization_server_policies(
            self, authServerId
    ):
        """
        Args:
            auth_server_id {str}
        Returns:
            list: Collection of Policy instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /policies
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(
                    find_policy_model(item["type"])(
                        self.form_response_body(item)
                        )
                    )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_authorization_server_policy(
            self, authServerId, policy
    ):
        """
        Args:
            auth_server_id {str}
            {policy}
        Returns:
            Policy
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /policies
            """)

        if isinstance(policy, dict):
            body = policy
        else:
            body = policy.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_authorization_server_policy(
            self, authServerId, policyId
    ):
        """
        Args:
            auth_server_id {str}
            policy_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /policies/{policyId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_authorization_server_policy(
            self, authServerId, policyId
    ):
        """
        Args:
            auth_server_id {str}
            policy_id {str}
        Returns:
            Policy
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /policies/{policyId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_authorization_server_policy(
            self, authServerId, policyId, policy
    ):
        """
        Args:
            auth_server_id {str}
            policy_id {str}
            {policy}
        Returns:
            Policy
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}
                /policies/{policyId}
            """)

        if isinstance(policy, dict):
            body = policy
        else:
            body = policy.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_o_auth_2_scopes(
            self, authServerId, query_params={}
    ):
        """
        Args:
            auth_server_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.q] {str}
            [query_params.filter] {str}
            [query_params.cursor] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of OAuth2Scope instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/scopes
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Scope)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(OAuth2Scope(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_o_auth_2_scope(
            self, authServerId, o_auth_2_scope
    ):
        """
        Args:
            auth_server_id {str}
            {o_auth_2_scope}
        Returns:
            OAuth2Scope
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/scopes
            """)

        if isinstance(o_auth_2_scope, dict):
            body = o_auth_2_scope
        else:
            body = o_auth_2_scope.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Scope)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Scope(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_o_auth_2_scope(
            self, authServerId, scopeId
    ):
        """
        Args:
            auth_server_id {str}
            scope_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/scopes
                /{scopeId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_o_auth_2_scope(
            self, authServerId, scopeId
    ):
        """
        Args:
            auth_server_id {str}
            scope_id {str}
        Returns:
            OAuth2Scope
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/scopes
                /{scopeId}
            """)

        body = {}
        headers = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Scope)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Scope(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_o_auth_2_scope(
            self, authServerId, scopeId, o_auth_2_scope
    ):
        """
        Args:
            auth_server_id {str}
            scope_id {str}
            {o_auth_2_scope}
        Returns:
            OAuth2Scope
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authorizationServers/{authServerId}/scopes
                /{scopeId}
            """)

        if isinstance(o_auth_2_scope, dict):
            body = o_auth_2_scope
        else:
            body = o_auth_2_scope.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, OAuth2Scope)

        if error:
            return (None, response, error)

        try:
            result = OAuth2Scope(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError


class TestEventHooksResource:
    """
    Integration Tests for the Event Hooks Resource
    """
    SDK_PREFIX = "python_sdk"
    EVENT_TYPE = "EVENT_TYPE"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_get_event_hook(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # create Event Hook
        event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate"],
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks",
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value"
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key",
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })

        created_event_hook, _, err = await client.create_event_hook(
            event_hook_model)
        # Verify details
        assert err is None
        assert isinstance(created_event_hook, models.EventHook)
        assert created_event_hook.id is not None
        assert created_event_hook.name == event_hook_model.name
        assert created_event_hook.events is not None
        assert len(created_event_hook.events.items) == 2
        assert created_event_hook.channel is not None
        assert created_event_hook.channel.config.uri ==\
            event_hook_model.channel.config.uri
        assert len(created_event_hook.channel.config.headers) == 1
        assert isinstance(created_event_hook.channel.config.headers[0],
                          models.EventHookChannelConfigHeader)

        # Retrieve
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name

        # Deactivate + Delete
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None
        _, err = await client.delete_event_hook(created_event_hook.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_event_hook(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # create Event Hook
        event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate"],
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks",
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value"
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key",
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })

        created_event_hook, _, err = await client.create_event_hook(
            event_hook_model)
        # Verify details
        assert err is None
        assert isinstance(created_event_hook, models.EventHook)
        assert created_event_hook.id is not None
        assert created_event_hook.name == event_hook_model.name
        assert created_event_hook.events is not None
        assert len(created_event_hook.events.items) == 2
        assert created_event_hook.channel is not None
        assert created_event_hook.channel.config.uri ==\
            event_hook_model.channel.config.uri

        # Retrieve
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name

        # Update
        updated_event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate",
                          "user.lifecycle.deactivate"],  # new
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks-UPDATE",  # new
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value updated"  # new
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key-updated",  # new
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })
        updated_event_hook, _, err = await client.update_event_hook(
            created_event_hook.id, updated_event_hook_model
        )
        assert err is None
        assert isinstance(updated_event_hook, models.EventHook)
        assert updated_event_hook.id == created_event_hook.id

        # Retrieve for validation
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == updated_event_hook.id
        assert retrieved_event_hook.name == updated_event_hook.name
        assert len(retrieved_event_hook.events.items) > len(
            created_event_hook.events.items)
        assert len(retrieved_event_hook.events.items) == 3
        assert "user.lifecycle.deactivate" in retrieved_event_hook.events.items
        assert retrieved_event_hook.channel.config.uri == \
            "https://www.example.com/eventHooks-UPDATE"

        # Deactivate + Delete
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None
        _, err = await client.delete_event_hook(created_event_hook.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_event_hook(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # create Event Hook
        event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate"],
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks",
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value"
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key",
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })

        created_event_hook, _, err = await client.create_event_hook(
            event_hook_model)
        # Verify details
        assert err is None
        assert isinstance(created_event_hook, models.EventHook)
        assert created_event_hook.id is not None
        assert created_event_hook.name == event_hook_model.name
        assert created_event_hook.events is not None
        assert len(created_event_hook.events.items) == 2
        assert created_event_hook.channel is not None
        assert created_event_hook.channel.config.uri ==\
            event_hook_model.channel.config.uri

        # Retrieve
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name

        # Deactivate + Delete
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None
        _, err = await client.delete_event_hook(created_event_hook.id)
        assert err is None

        # Retrieve for validation
        retrieved_event_hook, resp, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is not None
        assert isinstance(err, OktaAPIError)
        assert resp.get_status() == HTTPStatus.NOT_FOUND
        assert retrieved_event_hook is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_event_hooks(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        EVENT_HOOKS_COUNT = 3
        # Keep IDs
        created_event_hook_ids = []

        # create Event Hooks
        for index in range(EVENT_HOOKS_COUNT):
            event_hook_model = models.EventHook({
                "name": TestEventHooksResource.EVENT_TYPE + str(index),
                "events": models.EventSubscriptions({
                    "items": ["user.lifecycle.create",
                              "user.lifecycle.activate"],
                    "type": TestEventHooksResource.EVENT_TYPE
                }),
                "channel": models.EventHookChannel({
                    "type": "HTTP",
                    "version": "1.0.0",
                    "config": models.EventHookChannelConfig({
                        "uri": "https://www.example.com/eventHooks",
                        "headers": [
                            models.EventHookChannelConfigHeader({
                                "key": "X-Test-Header",
                                "value": "Test Header value"
                            })
                        ],
                        "authScheme": models.EventHookChannelConfigAuthScheme({
                            "key": "Authorization",
                            "value": "Test-API-Key",
                            "type": models.EventHookChannelConfigAuthSchemeType
                            .HEADER
                        })
                    })
                })
            })

            created_event_hook, _, err = await client.create_event_hook(
                event_hook_model)
            # Verify details
            assert err is None
            assert isinstance(created_event_hook, models.EventHook)
            assert created_event_hook.id is not None
            assert created_event_hook.name == event_hook_model.name
            assert created_event_hook.events is not None
            assert len(created_event_hook.events.items) == 2
            assert created_event_hook.channel is not None
            assert created_event_hook.channel.config.uri ==\
                event_hook_model.channel.config.uri
            created_event_hook_ids.append(created_event_hook.id)

        # List
        all_event_hooks, _, err = await client.list_event_hooks()
        assert err is None
        assert len(all_event_hooks) > 0
        assert len(all_event_hooks) == EVENT_HOOKS_COUNT
        for event_hook_id in created_event_hook_ids:
            assert((eh_id for eh_id in all_event_hooks
                    if eh_id.id == event_hook_id))

        # Deactivate + Delete
        for event_hook_id in created_event_hook_ids:
            _, _, err = await client.deactivate_event_hook(event_hook_id)
            assert err is None
            _, err = await client.delete_event_hook(event_hook_id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_deactivate_event_hook(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # create Event Hook
        event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate"],
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks",
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value"
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key",
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })

        created_event_hook, _, err = await client.create_event_hook(
            event_hook_model)
        # Verify details
        assert err is None
        assert isinstance(created_event_hook, models.EventHook)
        assert created_event_hook.id is not None
        assert created_event_hook.name == event_hook_model.name
        assert created_event_hook.events is not None
        assert len(created_event_hook.events.items) == 2
        assert created_event_hook.channel is not None
        assert created_event_hook.channel.config.uri ==\
            event_hook_model.channel.config.uri
        assert created_event_hook.status == "ACTIVE"

        # Retrieve
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name
        assert retrieved_event_hook.status == "ACTIVE"

        # Deactivate
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None

        # Verify
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name
        assert retrieved_event_hook.status == "INACTIVE"

        # Delete
        _, err = await client.delete_event_hook(created_event_hook.id)
        assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_activate_event_hook(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # create Event Hook
        event_hook_model = models.EventHook({
            "name": TestEventHooksResource.EVENT_TYPE,
            "events": models.EventSubscriptions({
                "items": ["user.lifecycle.create",
                          "user.lifecycle.activate"],
                "type": TestEventHooksResource.EVENT_TYPE
            }),
            "channel": models.EventHookChannel({
                "type": "HTTP",
                "version": "1.0.0",
                "config": models.EventHookChannelConfig({
                    "uri": "https://www.example.com/eventHooks",
                    "headers": [
                        models.EventHookChannelConfigHeader({
                            "key": "X-Test-Header",
                            "value": "Test Header value"
                        })
                    ],
                    "authScheme": models.EventHookChannelConfigAuthScheme({
                        "key": "Authorization",
                        "value": "Test-API-Key",
                        "type": models.EventHookChannelConfigAuthSchemeType
                        .HEADER
                    })
                })
            })
        })

        created_event_hook, _, err = await client.create_event_hook(
            event_hook_model)
        # Verify details
        assert err is None
        assert isinstance(created_event_hook, models.EventHook)
        assert created_event_hook.id is not None
        assert created_event_hook.name == event_hook_model.name
        assert created_event_hook.events is not None
        assert len(created_event_hook.events.items) == 2
        assert created_event_hook.channel is not None
        assert created_event_hook.channel.config.uri ==\
            event_hook_model.channel.config.uri
        assert created_event_hook.status == "ACTIVE"

        # Retrieve
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name
        assert retrieved_event_hook.status == "ACTIVE"

        # Deactivate
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None

        # Verify
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name
        assert retrieved_event_hook.status == "INACTIVE"

        # Activate
        _, _, err = await \
            client.activate_event_hook(created_event_hook.id)
        assert err is None

        # Verify
        retrieved_event_hook, _, err = await client.get_event_hook(
            created_event_hook.id)
        assert err is None
        assert isinstance(retrieved_event_hook, models.EventHook)
        assert retrieved_event_hook.id == created_event_hook.id
        assert retrieved_event_hook.name == created_event_hook.name
        assert retrieved_event_hook.status == "ACTIVE"

        # Deactivate
        _, _, err = await client.deactivate_event_hook(created_event_hook.id)
        assert err is None
        # Delete
        _, err = await client.delete_event_hook(created_event_hook.id)
        assert err is None
